@@ -347,6 +347,9 @@ bool pool_remove_sent_message(char kind, const char *name)
 	int i;
 	POOL_SENT_MESSAGE_LIST *msglist;
 
+	if (kind == 0 || name == NULL)
+		return false;
+
 	msglist = &pool_get_session_context(false)->message_list;
 
 	for (i = 0; i < msglist->size; i++)
@@ -575,6 +578,9 @@ POOL_SENT_MESSAGE *pool_get_sent_message(char kind, const char *name, POOL_SENT_
 
 	msglist = &pool_get_session_context(false)->message_list;
 
+	if (kind == 0 || name == NULL)
+		return NULL;
+
 	for (i = 0; i < msglist->size; i++)
 	{
 		if (msglist->sent_messages[i]->kind == kind &&
@@ -1154,8 +1160,9 @@ void pool_pending_message_add(POOL_PENDING_MESSAGE* message)
 
 	if (message->type != POOL_SYNC)
 		ereport(Elevel,
-				(errmsg("pool_pending_message_add: message type:%d message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
-						message->type, message->contents_len, message->query, message->statement, message->portal,
+				(errmsg("pool_pending_message_add: message type:%s message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
+						pool_pending_message_type_to_string(message->type),
+						message->contents_len, message->query, message->statement, message->portal,
 						message->node_ids[0], message->node_ids[1])));
 	else
 		ereport(Elevel,
@@ -1193,8 +1200,9 @@ POOL_PENDING_MESSAGE *pool_pending_message_head_message(void)
 	m = (POOL_PENDING_MESSAGE *) lfirst(cell);
 	message = copy_pending_message(m);
 	ereport(Elevel,
-			(errmsg("pool_pending_message_head_message: message type:%d message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
-					message->type, message->contents_len, message->query, message->statement, message->portal,
+			(errmsg("pool_pending_message_head_message: message type:%s message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
+					pool_pending_message_type_to_string(message->type),
+					message->contents_len, message->query, message->statement, message->portal,
 					message->node_ids[0], message->node_ids[1])));
 
 	MemoryContextSwitchTo(old_context);
@@ -1229,8 +1237,9 @@ POOL_PENDING_MESSAGE *pool_pending_message_pull_out(void)
 	m = (POOL_PENDING_MESSAGE *) lfirst(cell);
 	message = copy_pending_message(m);
 	ereport(Elevel,
-			(errmsg("pool_pending_message_pull_out: message type:%d message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
-					message->type, message->contents_len, message->query, message->statement, message->portal,
+			(errmsg("pool_pending_message_pull_out: message type:%s message len:%d query:%s statement:%s portal:%s node_ids[0]:%d node_ids[1]:%d",
+					pool_pending_message_type_to_string(message->type),
+					message->contents_len, message->query, message->statement, message->portal,
 					message->node_ids[0], message->node_ids[1])));
 
 	pool_pending_message_free_pending_message(m);
@@ -1376,6 +1385,90 @@ bool pool_pending_message_exists(void)
 	return list_length(session_context->pending_messages) > 0;
 }
 
+/*
+ * Convert enum pending message type to string. The returned string must not
+ * be modified or freed.
+ */
+const char *pool_pending_message_type_to_string(POOL_MESSAGE_TYPE type)
+{
+	static const char *pending_msg_string[] = {"Parse", "Bind", "Execute",
+											   "Descripbe", "Close", "Sync"};
+	if (type < 0 || type > POOL_SYNC)
+		return "unknown type";
+	return pending_msg_string[type];
+}
+
+/*
+ * Find the latest pending message having specified query context.  The
+ * returned message is a pointer to the message list. So do not free it using
+ * pool_pending_message_free_pending_message.
+ */
+POOL_PENDING_MESSAGE *pool_pending_message_find_lastest_by_query_context(POOL_QUERY_CONTEXT *qc)
+{
+	List *msgs;
+	POOL_PENDING_MESSAGE *msg;
+	int len;
+	ListCell *cell;
+
+	if (!session_context)
+	{
+		ereport(ERROR,
+				(errmsg("pool_pending_message_find_lastest_by_query_context: session context is not initialized")));
+	}
+
+	if (!session_context->pending_messages)
+		return NULL;
+
+	msgs = session_context->pending_messages;
+
+	len = list_length(msgs);
+	if (len <= 0)
+		return NULL;
+
+	ereport(DEBUG1,
+			(errmsg("pool_pending_message_find_lastest_by_query_context: num messages: %d",
+					len)));
+
+	while (len--)
+	{
+		cell = list_nth_cell(msgs, len);
+		if (cell)
+		{
+			msg = (POOL_PENDING_MESSAGE *) lfirst(cell);
+			if (msg->query_context == qc)
+			{
+				ereport(DEBUG1,
+						(errmsg("pool_pending_message_find_lastest_by_query_context: msg found. type: %s",
+								pool_pending_message_type_to_string(msg->type))));
+				return msg;
+			}
+			ereport(DEBUG1,
+					(errmsg("pool_pending_message_find_lastest_by_query_context: type: %s",
+							pool_pending_message_type_to_string(msg->type))));
+		}
+	}
+	return NULL;
+}
+
+/*
+ * Get target backend id from pending message assuming that the destination for
+ * the pending message is one of primary or standby node.
+ */
+int pool_pending_message_get_target_backend_id(POOL_PENDING_MESSAGE *msg)
+{
+	int backend_id = -1;
+
+	if (msg->node_ids[0] != -1)
+		backend_id = msg->node_ids[0];
+	else if (msg->node_ids[1] != -1)
+		backend_id = msg->node_ids[1];
+	else
+		ereport(ERROR,
+				(errmsg("pool_pending_message_get_target_backend_id: no target backend id found")));
+
+	return backend_id;
+}
+
 /*
  * Dump whole pending message list
  */
